﻿using Plane.Copters;
using Plane.Logging;
using GalaSoft.MvvmLight;
using Microsoft.Practices.ServiceLocation;
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Plane.FormationCreator.Formation
{
    public partial class FlightTask : ObservableObject
    {
        public FlightTask(FlightTaskType type)
        {
            TaskType = type;
        }

        private FlightTaskManager _flightTaskManager = ServiceLocator.Current.GetInstance<FlightTaskManager>();

        private ILogger _logger = ServiceLocator.Current.GetInstance<ILogger>();

        private FlightTaskStatus _Status = FlightTaskStatus.Stop;
        public FlightTaskStatus Status
        {
            get { return _Status; }
            set { Set(nameof(Status), ref _Status, value); }
        }

        private FlightTaskType _TaskType;
        public FlightTaskType TaskType
        {
            get { return _TaskType; }
            set
            {
                if (_TaskType != value)
                {
                    RaisePropertyChanged(nameof(TaskTypeIndex));
                }
                Set(nameof(TaskType), ref _TaskType, value);
            }
        }

        public int TaskTypeIndex
        {
            get { return (int)TaskType; }
            set
            {
                TaskType = (FlightTaskType)value;
                if ( (value == (int)FlightTaskType.Loiter) || (value == (int)FlightTaskType.Circle) )
                {
                    VerticalLift = true;
                }
            }
        }

        private bool _IsSelected;
        public bool IsSelected
        {
            get { return _IsSelected; }
            set { Set(nameof(IsSelected), ref _IsSelected, value); }
        }

		// 右键单击任务，用于隐藏任务图标, added by ZJF
        private bool _IsRightSelected;
        public bool IsRightSelected
        {
            get { return _IsRightSelected; }
            set { Set(nameof(IsRightSelected), ref _IsRightSelected, value); }
        }
		
        public List<FlightTaskSingleCopterInfo> SingleCopterInfos { get; set; } = new List<FlightTaskSingleCopterInfo>();

        private FlightTaskSingleCopterInfo _ModifyingSingleCopterInfo;
        public FlightTaskSingleCopterInfo ModifyingSingleCopterInfo
        {
            get { return _ModifyingSingleCopterInfo; }
            set { Set(nameof(ModifyingSingleCopterInfo), ref _ModifyingSingleCopterInfo, value); }
        }

        public async Task RunAsync()
        {
            switch (TaskType)
            {
                case FlightTaskType.FlyTo:
                    await RunFlyToTaskAsync().ConfigureAwait(false);
                    break;
                case FlightTaskType.Turn:
                    await RunTurnTaskAsync().ConfigureAwait(false);
                    break;
                case FlightTaskType.Circle:
                    await RunCircleTaskAsync().ConfigureAwait(false);
                    break;
                case FlightTaskType.SimpleCircle:
                    await RunSimpleCircleTaskAsync().ConfigureAwait(false);
                    break;
                case FlightTaskType.Loiter:
                    await RunLoiterTimeTaskAsync().ConfigureAwait(false);
                    break;
                case FlightTaskType.TakeOff:
                    //多架同时起飞
                    await MutilRunTakeOffTaskAsync().ConfigureAwait(false);
                    break;
                case FlightTaskType.ReturnToLand:   // Added by ZJF
                    //多架同时返航
                    await MutilRunReturnToLandTaskAsync().ConfigureAwait(false);
                    break;
                default:
                    throw new InvalidOperationException();
            }
        }
    }

    public enum FlightTaskType
    {
        FlyTo = 0,
        Turn = 1,
        Circle = 2,
        Loiter = 3,      // 加入悬停时间选项
        // LEDAction = 4,
        ReturnToLand = 4,    // added by ZJF. 20160315
        SimpleCircle = 5,
        TakeOff = 100
    }
}
