﻿using Plane.Copters;
using Plane.Geography;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Plane.Windows.Messages;

namespace Plane.FormationCreator.Formation
{
    public partial class FlightTask
    {
        private bool _StaggerRoutes = true;
        
        //是否有交错
        public bool StaggerRoutes
        {
            get { return _StaggerRoutes; }
            set { Set(nameof(StaggerRoutes), ref _StaggerRoutes, value); }
        }

        


        private bool _VerticalLift = false;
        public bool VerticalLift               // 垂直升降标志位，后面需要加入即使拖动地图上的飞机，也不会变化经纬度. added by ZJF
        {
            get { return _VerticalLift; }
            set
            {
                if (value)
                {
                    int currentIndex = _flightTaskManager.SelectedTaskIndex;
                    var currentCopterInfos = _flightTaskManager.SelectedTask.SingleCopterInfos;
                    var previousCopterInfos = _flightTaskManager.Tasks[currentIndex - 1].SingleCopterInfos;
                    for (int i = 0; i < currentCopterInfos.Count; i++)
                    {
                        currentCopterInfos[i].TargetLat = previousCopterInfos[i].TargetLat;
                        currentCopterInfos[i].TargetLng = previousCopterInfos[i].TargetLng;
                    }
                }
                
                Set(nameof(VerticalLift), ref _VerticalLift, value);
            }
        }

        


     

   

        public async Task RunFlyToTaskAsync()          // 全部飞到指定航点
        {
            //是否有交错
            if (StaggerRoutes)
            {
                var infos = SingleCopterInfos;
                var tasks = new Task[infos.Count];
                for (int i = 0; i < infos.Count; i++)
                {
                    var info = infos[i];
                    
                    tasks[i] = await Task.Factory.StartNew(async () =>
                    {
                        var internalInfo = info;
                          //if (i1 > 0)
                        //{
                        //    var prevCopter = infos[i1 - 1].Copter;
                        //    while (CheckCrossing(infos, i1) &&
                        //        prevCopter.Altitude - copter.Altitude < 2)
                        //    {
                        //        await Task.Delay(25).ConfigureAwait(false);
                        //    }
                        //}

                        await FlyToTaskFlySingleCopterAsync(internalInfo);
                    });
                }
                await Task.WhenAll(tasks).ConfigureAwait(false);
                
            }
            else
            {
                await Task.WhenAll(
                    SingleCopterInfos.Select(info => FlyToTaskFlySingleCopterAsync(info))
                ).ConfigureAwait(false);
            }
        }



        private async Task FlyToTaskFlySingleCopterAsync(FlightTaskSingleCopterInfo info)
        {
            int taskIndex = _flightTaskManager.CurrentRunningTaskIndex;
            int copterIndex = SingleCopterInfos.IndexOf(info);

            // 当该飞机被标记时，悬停并跳过飞行任务
            if ((bool)_copterManager.CopterStatus[copterIndex])
            {
                await info.Copter.HoverAsync();
                return;
            }

            await info.Copter.SetShowLEDAsync(info.FlytoShowLED);

            if (info.Copter.State != Plane.Copters.CopterState.CommandMode)
                await info.Copter.GuidAsync();

            DateTime dtNow = DateTime.Now;
            DateTime dtLastTime = DateTime.Now;
            TimeSpan ts = dtNow - dtLastTime;


         /*
             for (int i = 0; i < 5; i++)
            {
                await info.Copter.FlyToAsync(info.TargetLat, info.TargetLng, info.TargetAlt);
                await Task.Delay(10).ConfigureAwait(false);
            }
        */
           //发送目标航点1次
            await info.Copter.FlyToAsync(info.TargetLat, info.TargetLng, info.TargetAlt);
            await Task.Delay(10).ConfigureAwait(false);


            while (!info.Copter.ArrivedTarget(info.TargetLat, info.TargetLng, info.TargetAlt))
            {
                if (_flightTaskManager.IsPaused == true)
                {
                    await info.Copter.HoverAsync();
                    return;
                }
                await Task.Delay(250).ConfigureAwait(false); //判断是否到达位置10hz

                
                dtNow = DateTime.Now;
                ts = dtNow - dtLastTime;
                if (ts.TotalMilliseconds > 500)   // 每500ms发送一遍指点坐标
                {
                    for (int i = 0; i < 2; i++)
                    {
                        await info.Copter.FlyToAsync(info.TargetLat, info.TargetLng, info.TargetAlt);
                        await Task.Delay(10).ConfigureAwait(false);
                    }

                    dtLastTime = dtNow;
                }
                // 当该飞机被标记时，悬停并跳过飞行任务
                if ((bool)_copterManager.CopterStatus[copterIndex])
                {
                    await info.Copter.HoverAsync();
                    return;
                }
            }
            // await info.Copter.HoverAsync();
            if (taskIndex == _flightTaskManager.Tasks.Count() - 1)
            {
                await info.Copter.HoverAsync();
            }
        }
        

        private static bool CheckCrossing(List<FlightTaskSingleCopterInfo> infos, int currentIndex)
        {
            var info = infos[currentIndex];
            for (int i = 0; i < currentIndex; i++)
            {
                var nextInfo = infos[i];
                if (GeographyUtils.CheckCrossing2D(info.Copter.Latitude, info.Copter.Longitude, info.TargetLat, info.TargetLng,
                            nextInfo.Copter.Latitude, nextInfo.Copter.Longitude, nextInfo.TargetLat, nextInfo.TargetLng))
                {
                    return true;
                }
            }
            return false;
        }
    }
}
